@@ -729,623 +729,22 @@ void do_short(hd_data_t *hd_data, hd_t *hd, FILE *f)
 }
 
 
-#if 0
-typedef struct {
-  char *vendor, *model, *driver;
-} scanner_t;
-
-static scanner_t scanner_data[] = {
-  { "Abaton", "SCAN 300/GS", "abaton" },
-  { "Abaton", "SCAN 300/S", "abaton" },
-  { "Acer", "300f", "SnapScan" },
-  { "Acer", "310s", "SnapScan" },
-  { "Acer", "610plus", "SnapScan" },
-  { "Acer", "610s", "SnapScan" },
-  { "Acer", "Prisa 1240", "SnapScan" },
-  { "Acer", "Prisa 3300", "SnapScan" },
-  { "Acer", "Prisa 4300", "SnapScan" },
-  { "Acer", "Prisa 5300", "SnapScan" },
-  { "Acer", "Prisa 620s", "SnapScan" },
-  { "Acer", "Prisa 620u", "SnapScan" },
-  { "Acer", "Prisa 620ut", "SnapScan" },
-  { "Acer", "Prisa 640bu", "SnapScan" },
-  { "Acer", "Prisa 640u", "SnapScan" },
-  { "Agfa", "Arcus II", "microtek" },
-  { "Agfa", "DuoScan", "microtek" },
-  { "Agfa", "FOCUS COLOR", "agfafocus" },
-  { "Agfa", "FOCUS GS SCANNER", "agfafocus" },
-  { "Agfa", "FOCUS II", "agfafocus" },
-  { "Agfa", "FOCUS LINEART SCANNER", "agfafocus" },
-  { "Agfa", "SnapScan 1212u", "SnapScan" },
-  { "Agfa", "SnapScan 1236s", "SnapScan" },
-  { "Agfa", "SnapScan 1236u", "SnapScan" },
-  { "Agfa", "SnapScan 300", "SnapScan" },
-  { "Agfa", "SnapScan 310", "SnapScan" },
-  { "Agfa", "SnapScan 600", "SnapScan" },
-  { "Agfa", "SnapScan e20", "SnapScan" },
-  { "Agfa", "SnapScan e25", "SnapScan" },
-  { "Agfa", "SnapScan e40", "SnapScan" },
-  { "Agfa", "SnapScan e50", "SnapScan" },
-  { "Agfa", "SnapScan e60", "SnapScan" },
-  { "Agfa", "StudioScan", "microtek" },
-  { "Agfa", "StudioScan II", "microtek" },
-  { "Agfa", "StudioScan IIsi", "microtek" },
-  { "Apple", "APPLE SCANNER", "apple" },
-  { "Apple", "COLORONESCANNER", "apple" },
-  { "Apple", "ONESCANNER", "apple" },
-  { "Artec", "A6000C", "artec" },
-  { "Artec", "A6000C PLUS", "artec" },
-  { "Artec", "AM12S", "artec" },
-  { "Artec", "AT12", "artec" },
-  { "Artec", "AT3", "artec" },
-  { "Artec", "AT6", "artec" },
-  { "Artec", "ColorOneScanner", "artec" },
-  { "Avision", "AV 620 CS", "avision" },
-  { "Avision", "AV 6240", "avision" },
-  { "Avision", "AV 630 CS", "avision" },
-  { "B&H SCSI", "COPISCAN II 2135", "bh" },
-  { "B&H SCSI", "COPISCAN II 2137", "bh" },
-  { "B&H SCSI", "COPISCAN II 2137A", "bh" },
-  { "B&H SCSI", "COPISCAN II 2138A", "bh" },
-  { "B&H SCSI", "COPISCAN II 3238", "bh" },
-  { "B&H SCSI", "COPISCAN II 3338", "bh" },
-  { "B&H SCSI", "COPISCAN II 6338", "bh" },
-  { "BlackWidow", "BW4800SP", "artec" },
-  { "Canon", "CANOSCAN 2700F", "canon" },
-  { "Canon", "CANOSCAN 300", "canon" },
-  { "Canon", "CANOSCAN 600", "canon" },
-  { "Devcom", "9636PRO", "pie" },
-  { "Devcom", "9636S", "pie" },
-  { "EDGE", "KTX-9600US", "umax" },
-  { "Epson", "ES-8500", "epson" },
-  { "Epson", "EXPRESSION 1600", "epson" },
-  { "Epson", "EXPRESSION 1680", "epson" },
-  { "Epson", "EXPRESSION 636", "epson" },
-  { "Epson", "EXPRESSION 800", "epson" },
-  { "Epson", "FILMSCAN 200", "epson" },
-  { "Epson", "GT-5500", "epson" },
-  { "Epson", "GT-7000", "epson" },
-  { "Epson", "GT-8000", "epson" },
-  { "Epson", "PERFECTION 1200PHOTO", "epson" },
-  { "Epson", "PERFECTION 1200S", "epson" },
-  { "Epson", "PERFECTION 1200U", "epson" },
-  { "Epson", "PERFECTION 1240", "epson" },
-  { "Epson", "PERFECTION 1640", "epson" },
-  { "Epson", "PERFECTION 1650", "epson" },
-  { "Epson", "PERFECTION 610", "epson" },
-  { "Epson", "PERFECTION 636S", "epson" },
-  { "Epson", "PERFECTION 636U", "epson" },
-  { "Epson", "PERFECTION 640", "epson" },
-  { "Epson", "PERFECTION1200", "epson" },
-  { "Epson", "Perfection 600", "umax" },
-  { "Escom", "Image Scanner 256", "umax" },
-  { "Escort", "Galleria 600", "umax" },
-  { "Fujitsu", "M3091DCD", "m3091" },
-  { "Fujitsu", "M3096G", "m3096g" },
-  { "Fujitsu", "SP15C", "sp15c" },
-  { "Genius", "ColorPage-HR5 Pro", "umax" },
-  { "Guillemot", "Maxi Scan A4 Deluxe", "SnapScan" },
-  { "HP", "HP OFFICEJET K SERIES", "hp" },
-  { "HP", "HP OFFICEJET V SERIES", "hp" },
-  { "HP", "HP PHOTOSMART PHOTOSCANNER", "hp" },
-  { "HP", "HP PSC 700 SERIES", "hp" },
-  { "HP", "HP PSC 900 SERIES", "hp" },
-  { "HP", "HP SCANJET 3C", "hp" },
-  { "HP", "HP SCANJET 3P", "hp" },
-  { "HP", "HP SCANJET 4100C", "hp" },
-  { "HP", "HP SCANJET 4C", "hp" },
-  { "HP", "HP SCANJET 4P", "hp" },
-  { "HP", "HP SCANJET 5200C", "hp" },
-  { "HP", "HP SCANJET 6100C", "hp" },
-  { "HP", "HP SCANJET 6200C", "hp" },
-  { "HP", "HP SCANJET 6250C", "hp" },
-  { "HP", "HP SCANJET 6300C", "hp" },
-  { "HP", "HP SCANJET 6350C", "hp" },
-  { "HP", "HP SCANJET 6390C", "hp" },
-  { "HP", "HP SCANJET IIC", "hp" },
-  { "HP", "HP SCANJET IICX", "hp" },
-  { "HP", "HP SCANJET IIP", "hp" },
-  { "HP", "HP ScanJet 5p", "hp" },
-  { "HP", "HP4200", "hp4200" },
-  { "Highscreen", "Scanboostar Premium", "umax" },
-  { "Linotype Hell", "Jade", "umax" },
-  { "Linotype Hell", "Jade2", "umax" },
-  { "Linotype Hell", "Linoscan 1400", "umax" },
-  { "Linotype Hell", "Opal", "umax" },
-  { "Linotype Hell", "Opal Ultra", "umax" },
-  { "Linotype Hell", "Saphir", "umax" },
-  { "Linotype Hell", "Saphir HiRes", "umax" },
-  { "Linotype Hell", "Saphir Ultra", "umax" },
-  { "Linotype Hell", "Saphir Ultra II", "umax" },
-  { "Linotype Hell", "Saphir2", "umax" },
-  { "Microtek", "Phantom 636", "microtek2" },
-  { "Microtek", "ScanMaker 330", "microtek2" },
-  { "Microtek", "ScanMaker 3600", "sm3600" },
-  { "Microtek", "ScanMaker 630", "microtek2" },
-  { "Microtek", "ScanMaker 636", "microtek2" },
-  { "Microtek", "ScanMaker 9600XL", "microtek2" },
-  { "Microtek", "ScanMaker E3plus", "microtek2" },
-  { "Microtek", "ScanMaker V300", "microtek2" },
-  { "Microtek", "ScanMaker V310", "microtek2" },
-  { "Microtek", "ScanMaker V600", "microtek2" },
-  { "Microtek", "ScanMaker V6USL", "microtek2" },
-  { "Microtek", "ScanMaker X6", "microtek2" },
-  { "Microtek", "ScanMaker X6EL", "microtek2" },
-  { "Microtek", "ScanMaker X6USB", "microtek2" },
-  { "Microtek", "Scanmaker 35", "microtek" },
-  { "Microtek", "Scanmaker 35t+", "microtek" },
-  { "Microtek", "Scanmaker 45t", "microtek" },
-  { "Microtek", "Scanmaker 600G", "microtek" },
-  { "Microtek", "Scanmaker 600G S", "microtek" },
-  { "Microtek", "Scanmaker 600GS", "microtek" },
-  { "Microtek", "Scanmaker 600S", "microtek" },
-  { "Microtek", "Scanmaker 600Z", "microtek" },
-  { "Microtek", "Scanmaker 600Z S", "microtek" },
-  { "Microtek", "Scanmaker 600ZS", "microtek" },
-  { "Microtek", "Scanmaker E2", "microtek" },
-  { "Microtek", "Scanmaker E3", "microtek" },
-  { "Microtek", "Scanmaker E6", "microtek" },
-  { "Microtek", "Scanmaker II", "microtek" },
-  { "Microtek", "Scanmaker IIG", "microtek" },
-  { "Microtek", "Scanmaker IIHR", "microtek" },
-  { "Microtek", "Scanmaker III", "microtek" },
-  { "Microtek", "Scanmaker IISP", "microtek" },
-  { "Microtek", "SlimScan C6", "microtek2" },
-  { "Mustek", "1200 CU", "mustek_usb" },
-  { "Mustek", "1200 CU Plus", "mustek_usb" },
-  { "Mustek", "1200 UB", "mustek_usb" },
-  { "Mustek", "600 CU", "mustek_usb" },
-  { "Mustek", "Paragon 1200 A3 Pro", "mustek" },
-  { "Mustek", "Paragon 1200 III SP", "mustek" },
-  { "Mustek", "Paragon 1200 LS", "mustek" },
-  { "Mustek", "Paragon 1200 SP Pro", "mustek" },
-  { "Mustek", "Paragon 600 II CD", "mustek" },
-  { "Mustek", "Paragon 800 II SP", "mustek" },
-  { "Mustek", "Paragon MFC-600S", "mustek" },
-  { "Mustek", "Paragon MFC-800S", "mustek" },
-  { "Mustek", "Paragon MFS-12000CX", "mustek" },
-  { "Mustek", "Paragon MFS-12000SP", "mustek" },
-  { "Mustek", "Paragon MFS-1200SP", "mustek" },
-  { "Mustek", "Paragon MFS-6000CX", "mustek" },
-  { "Mustek", "Paragon MFS-6000SP", "mustek" },
-  { "Mustek", "Paragon MFS-8000SP", "mustek" },
-  { "Mustek", "ScanExpress 12000SP", "mustek" },
-  { "Mustek", "ScanExpress 12000SP Plus", "mustek" },
-  { "Mustek", "ScanExpress 6000SP", "mustek" },
-  { "Mustek", "ScanExpress A3 SP", "mustek" },
-  { "Mustek", "ScanMagic 600 II SP", "mustek" },
-  { "Mustek", "ScanMagic 9636S", "mustek" },
-  { "Mustek", "ScanMagic 9636S Plus", "mustek" },
-  { "NEC", "PC-IN500/4C", "nec" },
-  { "Nikon", "AX-210", "umax" },
-  { "Nikon", "LS-1000", "coolscan" },
-  { "Nikon", "LS-20", "coolscan" },
-  { "Nikon", "LS-2000", "coolscan" },
-  { "Nikon", "LS-30", "coolscan" },
-  { "Pie", "9630S", "pie" },
-  { "Pie", "ScanAce 1230S", "pie" },
-  { "Pie", "ScanAce 1236S", "pie" },
-  { "Pie", "ScanAce 630S", "pie" },
-  { "Pie", "ScanAce 636S", "plustek" },
-  { "Pie", "ScanAce II", "pie" },
-  { "Pie", "ScanAce II Plus", "pie" },
-  { "Pie", "ScanAce III", "pie" },
-  { "Pie", "ScanAce III Plus", "pie" },
-  { "Pie", "ScanAce Plus", "pie" },
-  { "Pie", "ScanAce ScanMedia", "pie" },
-  { "Pie", "ScanAce ScanMedia II", "pie" },
-  { "Pie", "ScanAce V", "pie" },
-  { "Plustek", "OpticPro 19200S", "artec" },
-  { "Polaroid", "DMC", "dmc" },
-  { "Ricoh", "Ricoh IS50", "ricoh" },
-  { "Ricoh", "Ricoh IS60", "ricoh" },
-  { "Scanport", "SQ4836", "microtek2" },
-  { "Sharp", "9036 Flatbed scanner", "sharp" },
-  { "Sharp", "JX-250", "sharp" },
-  { "Sharp", "JX-320", "sharp" },
-  { "Sharp", "JX-330", "sharp" },
-  { "Sharp", "JX-350", "sharp" },
-  { "Sharp", "JX-610", "sharp" },
-  { "Siemens", "9036 Flatbed scanner", "s9036" },
-  { "Siemens", "FOCUS COLOR PLUS", "agfafocus" },
-  { "Siemens", "ST400", "st400" },
-  { "Siemens", "ST800", "st400" },
-  { "Tamarack", "Artiscan 12000C", "tamarack" },
-  { "Tamarack", "Artiscan 6000C", "tamarack" },
-  { "Tamarack", "Artiscan 8000C", "tamarack" },
-  { "Trust", "Compact Scan USB 19200", "mustek_usb" },
-  { "Trust", "Imagery 1200 SP", "mustek" },
-  { "Trust", "Imagery 4800 SP", "mustek" },
-  { "Trust", "SCSI Connect 19200", "mustek" },
-  { "Trust", "SCSI excellence series 19200", "mustek" },
-  { "UMAX", "Astra 1200S", "umax" },
-  { "UMAX", "Astra 1220S", "umax" },
-  { "UMAX", "Astra 2100S", "umax" },
-  { "UMAX", "Astra 2200", "umax" },
-  { "UMAX", "Astra 2200 S", "umax" },
-  { "UMAX", "Astra 2200 U", "umax" },
-  { "UMAX", "Astra 2400S", "umax" },
-  { "UMAX", "Astra 600S", "umax" },
-  { "UMAX", "Astra 610S", "umax" },
-  { "UMAX", "Gemini D-16", "umax" },
-  { "UMAX", "Mirage D-16L", "umax" },
-  { "UMAX", "Mirage II", "umax" },
-  { "UMAX", "Mirage IIse", "umax" },
-  { "UMAX", "PL-II", "umax" },
-  { "UMAX", "PSD", "umax" },
-  { "UMAX", "PowerLook", "umax" },
-  { "UMAX", "PowerLook 2000", "umax" },
-  { "UMAX", "PowerLook 3000", "umax" },
-  { "UMAX", "PowerLook III", "umax" },
-  { "UMAX", "Supervista S-12", "umax" },
-  { "UMAX", "UC 1200S", "umax" },
-  { "UMAX", "UC 1200SE", "umax" },
-  { "UMAX", "UC 1260", "umax" },
-  { "UMAX", "UC 630", "umax" },
-  { "UMAX", "UC 840", "umax" },
-  { "UMAX", "UG 630", "umax" },
-  { "UMAX", "UG 80", "umax" },
-  { "UMAX", "UMAX S-12", "umax" },
-  { "UMAX", "UMAX S-12G", "umax" },
-  { "UMAX", "UMAX S-6E", "umax" },
-  { "UMAX", "UMAX S-6EG", "umax" },
-  { "UMAX", "UMAX VT600", "umax" },
-  { "UMAX", "Vista S6", "umax" },
-  { "UMAX", "Vista S6E", "umax" },
-  { "UMAX", "Vista-S8", "umax" },
-  { "UMAX", "Vista-T630", "umax" },
-  { "Ultima", "A6000C", "artec" },
-  { "Ultima", "A6000C PLUS", "artec" },
-  { "Ultima", "AM12S", "artec" },
-  { "Ultima", "AT12", "artec" },
-  { "Ultima", "AT3", "artec" },
-  { "Ultima", "AT6", "artec" },
-  { "Ultima", "ColorOneScanner", "artec" },
-  { "Vobis", "HighScan", "microtek2" },
-  { "Vobis", "Scanboostar Premium", "umax" },
-  { "Vuego", "Close SnapScan 310 compatible.", "SnapScan" }
-};
-
-static char *scanner_info(hd_t *hd)
-{
-  int i;
-
-  if(!hd->vendor.name || !hd->device.name) return NULL;
-
-  for(i = 0; (unsigned) i < sizeof scanner_data / sizeof *scanner_data; i++) {
-    if(
-      !strcasecmp(scanner_data[i].vendor, hd->vendor.name) &&
-      !strcasecmp(scanner_data[i].model, hd->device.name)
-    ) {
-      return scanner_data[i].driver;
-    }
-  }
-
-  return NULL;
-}
-
-#endif
-
 void do_test(hd_data_t *hd_data)
 {
-#if 0
-  hd_t *hd, *hd0;
-  hd_res_t *res;
-  driver_info_t *di;
-  FILE *f;
-  int i, wheels, buttons;
-  unsigned u;
-  uint64_t ul;
-  char *s, *s1;
-  hd_hw_item_t item, items[] = {
-    hw_display, hw_monitor, hw_tv, hw_sound, hw_mouse, hw_disk, hw_cdrom,
-    hw_floppy, hw_modem, hw_isdn, hw_scanner, hw_camera
+  hd_t *hd;
+  char buf[10];
+  hd_hw_item_t hw_items[] = {
+    hw_sound, 0
   };
 
-  hd_set_probe_feature(hd_data, pr_default);
-  hd_scan(hd_data);
-
-  f = fopen("/tmp/hw_overview.log", "w");
-
-  for(i = 0; (unsigned) i < sizeof items / sizeof *items; i++) {
-    item = items[i];
-    hd0 = hd_list(hd_data, item, 0, NULL);
-
-    if(!hd0) continue;
-  
-    switch(item) {
-      case hw_disk:
-        fprintf(f, "Disk\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          u = 0;
-          for(res = hd->res; res; res = res->next) {
-            if(res->any.type == res_size && res->size.unit == size_unit_sectors) {
-              ul = (uint64_t) res->size.val1 * (res->size.val2 ?: 0x200);
-              u = ((ul >> 29) + 1) >> 1;
-            }
-          }
-          s = hd->bus.name;
-          fprintf(f, "  %s", hd->model);
-          if(u) {
-            fprintf(f, " (");
-            if(s) fprintf(f, "%s, ", s);
-            fprintf(f, "%u GB)", u);
-          }
-          fprintf(f, "\n");
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_cdrom:
-        fprintf(f, "CD-ROM\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          s = hd->bus.name;
-          fprintf(f, "  %s (", hd->model);
-          if(s) fprintf(f, "%s, ", s);
-          fprintf(f, "%s)", hd->prog_if.name ?: "CD-ROM");
-          fprintf(f, "\n");
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_monitor:
-        fprintf(f, "Monitor\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          s = hd->model;
-          if(!strcmp(hd->unique_id, "rdCR.EY_qmtb9YY0")) s = "not detected";
-          fprintf(f, "  %s\n", s);
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_display:
-        fprintf(f, "GFX Card\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          u = 0;
-          s1 = NULL;
-          for(di = hd->driver_info; di; di = di->next) {
-            if(di->any.type == di_x11) {
-              if(!s1) s1 = di->x11.server;
-              if(di->x11.x3d && !u) {
-                s1 = di->x11.server;
-                u = 1;
-              }
-            }
-          }
-          if(!s1) {
-            s1 = "not supported";
-            u = 0;
-          }
-          fprintf(f, "  %s (%s", hd->model, s1);
-          if(u) fprintf(f, ", 3D support");
-          fprintf(f, ")");
-          fprintf(f, "\n");
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_mouse:
-        fprintf(f, "Mouse\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          buttons = wheels = -1;	// make gcc happy
-          s = NULL;
-          for(di = hd->driver_info; di; di = di->next) {
-            if(di->any.type == di_mouse) {
-              buttons = di->mouse.buttons;
-              wheels = di->mouse.wheels;
-              s = di->mouse.xf86;
-              break;
-            }
-          }
-          if(!s) {
-            s = "not supported";
-            buttons = wheels = -1;
-          }
-          fprintf(f, "  %s (%s", hd->model, s);
-          if(buttons >= 0) fprintf(f, ", %d buttons", buttons);
-          if(wheels >= 0) fprintf(f, ", %d wheels", wheels);
-          fprintf(f, ")");
-          fprintf(f, "\n");
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_tv:
-        fprintf(f, "TV Card\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          s = NULL;
-          for(di = hd->driver_info; di; di = di->next) {
-            if(
-              (di->any.type == di_any || di->any.type == di_module) &&
-              di->any.hddb0 &&
-              di->any.hddb0->str
-            ) {
-              s = di->any.hddb0->str;
-              break;
-            }
-          }
-          if(!s) {
-            s = "not supported";
-          }
-          fprintf(f, "  %s (%s)\n", hd->model, s);
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_sound:
-        fprintf(f, "Sound Card\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          s = NULL;
-          for(di = hd->driver_info; di; di = di->next) {
-            if(
-              (di->any.type == di_any || di->any.type == di_module) &&
-              di->any.hddb0 &&
-              di->any.hddb0->str
-            ) {
-              s = di->any.hddb0->str;
-              break;
-            }
-          }
-          if(!s) {
-            s = "not supported";
-          }
-          fprintf(f, "  %s (%s)\n", hd->model, s);
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_camera:
-        fprintf(f, "Digital Camera/WebCam\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          fprintf(f, "  %s\n", hd->model);
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_floppy:
-        fprintf(f, "Floppy/Zip Drive\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          fprintf(f, "  %s\n", hd->model);
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_modem:
-        fprintf(f, "Modem\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          fprintf(f, "  %s\n", hd->model);
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_isdn:
-        fprintf(f, "ISDN\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          fprintf(f, "  %s (%ssupported)\n", hd->model, hd->driver_info ? "" : "not ");
-        }
-        fprintf(f, "\n");
-        break;
-
-      case hw_scanner:
-        fprintf(f, "Scanner\n");
-        for(hd = hd0; hd; hd = hd->next) {
-          s = scanner_info(hd);
-          if(!s) s = "not supported";
-          fprintf(f, "  %s (%s)\n", hd->model, s);
-        }
-        fprintf(f, "\n");
-        break;
-
-      default:
-        break;
-    }
-
-    hd_free_hd_list(hd0);
-
+  for(hd = hd_list2(hd_data, hw_items, 1); hd; hd = hd->next) {
+    do {
+      *buf = 0;
+      fgets(buf, sizeof buf, stdin);
+      hd_add_driver_data(hd_data, hd);
+      hd_dump_entry(hd_data, hd, stderr);
+    } while(*buf == '\n');
   }
-
-  fclose(f);
-
-  f = fopen("/tmp/hw_detail.log", "w");
-
-  if(hd_data->log) {
-    fprintf(f,
-      "============ start detailed hardware log ============\n"
-    );
-    fprintf(f,
-      "============ start debug info ============\n%s=========== end debug info ============\n",
-      hd_data->log
-    );
-  }
-
-  for(hd = hd_data->hd; hd; hd = hd->next) {
-    hd_dump_entry(hd_data, hd, f);
-  }
-
-  fprintf(f,
-    "============ end detailed hardware log ============\n"
-  );
-
-  fclose(f);
-
-  fprintf(stderr, "\n");
-
-#endif
-
-#if 0
-  hd_t *hd;
-  hd_t *hd0 = NULL;
-
-  for(hd = hd_list(hd_data, hw_cdrom, 1, hd0); hd; hd = hd->next) {
-    fprintf(stderr, "cdrom: %s, %s\n", hd->unix_dev_name, hd->model);
-  }
-
-  for(hd = hd_list(hd_data, hw_cdrom, 1, hd0); hd; hd = hd->next) {
-    fprintf(stderr, "cdrom: %s, %s\n", hd->unix_dev_name, hd->model);
-  }
-#endif
-
-#if 0
-  hd_t *hd;
-
-  hd = hd_list(hd_data, hw_disk, 1, NULL);
-  hd_free_hd_list(hd);
-  hd_free_hd_data(hd_data);
-
-  hd = hd_list(hd_data, hw_cdrom, 1, NULL);
-  hd_free_hd_list(hd);
-  hd_free_hd_data(hd_data);
-
-  hd = hd_list(hd_data, hw_storage_ctrl, 1, NULL);
-  hd_free_hd_list(hd);
-  hd_free_hd_data(hd_data);
-
-  hd = hd_list(hd_data, hw_display, 1, NULL);
-  hd_free_hd_list(hd);
-  hd_free_hd_data(hd_data);
-
-#if 0
-  for(hd = hd_data->hd; hd; hd = hd->next) {
-    hd_dump_entry(hd_data, hd, stdout);
-  }
-
-  printf("%s\n", hd_data->log);
-#endif
-
-#endif
-
-#if 0
-  hd_t *hd, *hd0;
-
-  hd0 = hd_list(hd_data, hw_sound, 1, NULL);
-  hd0 = hd_list(hd_data, hw_sound, 1, NULL);
-
-  for(hd = hd0; hd; hd = hd->next) {
-    hd_dump_entry(hd_data, hd, stdout);
-  }
-
-#if 0
-  hd_data->log = free_mem(hd_data->log);
-  dump_hddb_data(hd_data, hd_data->hddb_dev, "hddb_dev, final");  
-  if(hd_data->log) printf("%s", hd_data->log);
-#endif
-
-#endif
-
-#if 0
-  hd_t *hd;
-  driver_info_t *di;
-  str_list_t *sl;
-
-  hd = hd_list(hd_data, hw_sys, 1, NULL);
-
-  for(di = hd->driver_info; di; di = di->next) {
-    if(di->any.type == di_module) {
-      for(sl = di->module.names; sl; sl = sl->next) printf("%s\n", sl->str);
-    }
-  }
-
-#endif
 }
 
 
