@@ -155,7 +155,7 @@ void hd_scan_bios(hd_data_t *hd_data)
   vbe_info_t *vbe;
   vbe_mode_info_t *mi;
   hd_res_t *res;
-  str_list_t *sl;
+  str_list_t *sl, *sl0;
 #endif
 
   if(!hd_probe_feature(hd_data, pr_bios)) return;
@@ -526,6 +526,18 @@ void hd_scan_bios(hd_data_t *hd_data)
     }
   }
 
+  if(hd_probe_feature(hd_data, pr_bios_acpi)) {
+    PROGRESS(6, 0, "acpi");
+
+    if((sl0 = read_file("|/usr/sbin/acpidump 2>/dev/null", 0, 0))) {
+      ADD2LOG("----- %s -----\n", "ACPI dump");
+      for(sl = sl0; sl; sl = sl->next) {
+        ADD2LOG("%s", sl->str);
+      }
+      ADD2LOG("----- %s end -----\n", "ACPI dump");
+      free_str_list(sl0);
+    }
+  }
 }
 
 
