@@ -117,7 +117,8 @@ typedef enum probe_feature {
   pr_block, pr_block_cdrom, pr_block_part, pr_edd, pr_edd_mod, pr_bios_ddc,
   pr_bios_fb, pr_bios_mode, pr_input, pr_block_mods, pr_bios_vesa,
   pr_cpuemu_debug, pr_scsi_noserial, pr_wlan, pr_bios_crc, pr_hal,
-  pr_bios_vram,
+  pr_bios_vram, pr_bios_acpi, pr_bios_ddc_ports_1, pr_bios_ddc_ports_2,
+  pr_bios_ddc_ports_3, pr_bios_ddc_ports_4, pr_modules_pata,
   pr_max, pr_lxrc, pr_default, 
   pr_all		/**< pr_all must be last */
 } hd_probe_feature_t;
@@ -491,7 +492,8 @@ typedef struct {
   unsigned modes;		/**< number of supported video modes */
   vbe_mode_info_t *mode;	/**< video mode list */
   unsigned current_mode;	/**< current video mode */
-  unsigned char ddc_port[2][0x80];	/**< ddc monitor info per port */
+  unsigned ddc_ports;		/**< max ports to probe */
+  unsigned char ddc_port[4][0x80];	/**< ddc monitor info per port */
 } vbe_info_t;
 
 
@@ -1379,6 +1381,7 @@ typedef struct {
   char *model_name;		/**< axp: cpu model */
   char *platform;		/**< x86: NULL */
   str_list_t *features;		/**< x86: flags */
+  double bogo;			/**< bogo mips */
 } cpu_info_t;
 
 
@@ -2388,6 +2391,11 @@ typedef struct s_hd_t {
    */
   char *driver;
 
+  /**
+   * Currently active driver module (if any).
+   */
+  char *driver_module;
+
   /**
    * List of currently active drivers.
    */
@@ -2513,6 +2521,7 @@ typedef struct {
     unsigned nobioscrc:1;	/**< internal: don't check VBIOS crc */
     unsigned biosvram:1;	/**< internal: map Video BIOS RAM (128k at 0xa0000) */
     unsigned nowpa:1;           /**< do not probe WPA capabilities */
+    unsigned pata:1;            /**< use new libata modules instead of classical ide modules */
   } flags;
 
 
@@ -2622,6 +2631,8 @@ hd_t *hd_list_with_status(hd_data_t *hd_data, hd_hw_item_t item, hd_status_t sta
 hd_t *hd_list2(hd_data_t *hd_data, hd_hw_item_t *items, int rescan);
 hd_t *hd_list_with_status2(hd_data_t *hd_data, hd_hw_item_t *items, hd_status_t status);
 
+void hd_add_driver_data(hd_data_t *hd_data, hd_t *hd);
+
 int hd_has_pcmcia(hd_data_t *hd_data);
 #if 0
 /**
