@@ -23,15 +23,20 @@
 #include "hd_int.h"
 #include "net.h"
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- * gather network interface info
+/**
+ * @defgroup NETint Network devices
+ * @ingroup libhdDEVint
+ * @brief Network device scan functions
  *
- * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ * Gather network interface info
+ *
+ * @{
  */
 
 static void get_driverinfo(hd_data_t *hd_data, hd_t *hd);
 static void get_linkstate(hd_data_t *hd_data, hd_t *hd);
 static void add_xpnet(hd_data_t *hdata);
+static void add_mv(hd_data_t *hdata);
 static void add_iseries(hd_data_t *hdata);
 static void add_uml(hd_data_t *hdata);
 static void add_kma(hd_data_t *hdata);
@@ -50,12 +55,9 @@ void hd_scan_net(hd_data_t *hd_data)
   char *s, *hw_addr;
   hd_res_t *res, *res1;
   uint64_t ul0;
-
-  struct sysfs_class *sf_class;
-  struct sysfs_class_device *sf_cdev;
-  struct sysfs_device *sf_dev;
-  struct sysfs_driver *sf_drv;
-  struct dlist *sf_cdev_list;
+  str_list_t *sf_class, *sf_class_e;
+  char *sf_cdev = NULL, *sf_dev = NULL;
+  char *sf_drv_name, *sf_drv;
 
   if(!hd_probe_feature(hd_data, pr_net)) return;
 
@@ -67,47 +69,50 @@ void hd_scan_net(hd_data_t *hd_data)
 
   PROGRESS(1, 0, "get network data");
 
-  sf_class = sysfs_open_class("net");
+  sf_class = reverse_str_list(read_dir("/sys/class/net", 'd'));
 
   if(!sf_class) {
     ADD2LOG("sysfs: no such class: net\n");
     return;
   }
 
-  sf_cdev_list = sysfs_get_class_devices(sf_class);
-  if(sf_cdev_list) dlist_for_each_data(sf_cdev_list, sf_cdev, struct sysfs_class_device) {
+  for(sf_class_e = sf_class; sf_class_e; sf_class_e = sf_class_e->next) {
+    str_printf(&sf_cdev, 0, "/sys/class/net/%s", sf_class_e->str);
+
     hd_card = NULL;
 
     ADD2LOG(
-      "  net interface: name = %s, classname = %s, path = %s\n",
-      sf_cdev->name,
-      sf_cdev->classname,
-      hd_sysfs_id(sf_cdev->path)
+      "  net interface: name = %s, path = %s\n",
+      sf_class_e->str,
+      hd_sysfs_id(sf_cdev)
     );
 
     if_type = -1;
-    if(hd_attr_uint(sysfs_get_classdev_attr(sf_cdev, "type"), &ul0, 0)) {
+    if(hd_attr_uint(get_sysfs_attr_by_path(sf_cdev, "type"), &ul0, 0)) {
       if_type = ul0;
       ADD2LOG("    type = %d\n", if_type);
     }
 
     hw_addr = NULL;
-    if((s = hd_attr_str(sysfs_get_classdev_attr(sf_cdev, "address")))) {
+    if((s = get_sysfs_attr_by_path(sf_cdev, "address"))) {
       hw_addr = canon_str(s, strlen(s));
       ADD2LOG("    hw_addr = %s\n", hw_addr);
     }
 
-    sf_dev = sysfs_get_classdev_device(sf_cdev);
+    sf_dev = new_str(hd_read_sysfs_link(sf_cdev, "device"));
     if(sf_dev) {
-      ADD2LOG("    net device: path = %s\n", hd_sysfs_id(sf_dev->path));
+      ADD2LOG("    net device: path = %s\n", hd_sysfs_id(sf_dev));
     }
 
-    sf_drv = sysfs_get_classdev_driver(sf_cdev);
+    sf_drv_name = NULL;
+    sf_drv = hd_read_sysfs_link(sf_dev, "driver");
     if(sf_drv) {
+      sf_drv_name = strrchr(sf_drv, '/');
+      if(sf_drv_name) sf_drv_name++;
       ADD2LOG(
         "    net driver: name = %s, path = %s\n",
-        sf_drv->name,
-        hd_sysfs_id(sf_drv->path)
+        sf_drv_name,
+        hd_sysfs_id(sf_drv)
       );
     }
 
@@ -125,20 +130,20 @@ void hd_scan_net(hd_data_t *hd_data)
 
     hw_addr = free_mem(hw_addr);
 
-    hd->unix_dev_name = new_str(sf_cdev->name);
-    hd->sysfs_id = new_str(hd_sysfs_id(sf_cdev->path));
+    hd->unix_dev_name = new_str(sf_class_e->str);
+    hd->sysfs_id = new_str(hd_sysfs_id(sf_cdev));
 
-    if(sf_drv) {
-      add_str_list(&hd->drivers, sf_drv->name);
+    if(sf_drv_name) {
+      add_str_list(&hd->drivers, sf_drv_name);
     }
     else if(hd->res) {
       get_driverinfo(hd_data, hd);
     }
 
     if(sf_dev) {
-      hd->sysfs_device_link = new_str(hd_sysfs_id(sf_dev->path)); 
+      hd->sysfs_device_link = new_str(hd_sysfs_id(sf_dev)); 
 
-      hd_card = hd_find_sysfs_id(hd_data, hd_sysfs_id(sf_dev->path));
+      hd_card = hd_find_sysfs_id(hd_data, hd_sysfs_id(sf_dev));
       if(hd_card) {
         hd->attached_to = hd_card->idx;
 
@@ -180,6 +185,7 @@ void hd_scan_net(hd_data_t *hd_data)
     "xp"	sc_nif_xp
     "usb"	sc_nif_usb
 #endif
+
     switch(if_type) {
       case ARPHRD_ETHER:	/* eth */
         hd->sub_class.id = sc_nif_ethernet;
@@ -281,15 +287,19 @@ void hd_scan_net(hd_data_t *hd_data)
         }
       }
     }
+
+    sf_dev = free_mem(sf_dev);
   }
 
-  sysfs_close_class(sf_class);
+  sf_cdev = free_mem(sf_cdev);
+  sf_class = free_str_list(sf_class);
 
   if(hd_is_sgi_altix(hd_data)) add_xpnet(hd_data);
   if(hd_is_iseries(hd_data)) add_iseries(hd_data);
   add_uml(hd_data);
   add_xen(hd_data);
   add_kma(hd_data);
+  add_mv(hd_data);
 
   /* add link status info */
   for(hd = hd_data->hd ; hd; hd = hd->next) {
@@ -435,6 +445,90 @@ void add_xpnet(hd_data_t *hd_data)
 }
 
 
+/*
+ * Marvell Gigabit Ethernet thing
+ */
+void add_mv(hd_data_t *hd_data)
+{
+  hd_t *hd, *hd_card;
+  hd_res_t *res, *res2;
+  struct stat sbuf;
+  char *s, *sf_dev, *sf_dev_name, *module;
+
+  /*
+   * Actually there are two (.0 & .1), but only one seems to be used - so we
+   * don't care.
+   */
+  sf_dev = "/sys/devices/platform/mv643xx_eth.0";
+  sf_dev_name = "mv643xx_eth.0";
+  module = "mv643xx_eth";
+
+  if(stat(sf_dev, &sbuf) || !S_ISDIR(sbuf.st_mode)) return;
+
+  for(hd = hd_data->hd ; hd; hd = hd->next) {
+    if(
+      hd->vendor.id == MAKE_ID(TAG_PCI, 0x11ab) &&
+      hd->device.id == MAKE_ID(TAG_PCI, 0x6460)
+    ) break;
+  }
+
+  hd_card = add_hd_entry(hd_data, __LINE__, 0);
+  hd_card->base_class.id = bc_network;
+  hd_card->sub_class.id = 0;
+
+  hd_card->sysfs_id = new_str(hd_sysfs_id(sf_dev));
+  hd_card->sysfs_bus_id = new_str(sf_dev_name);
+
+  if((s = get_sysfs_attr_by_path(sf_dev, "modalias"))) {
+    hd_card->modalias = canon_str(s, strlen(s));
+    ADD2LOG("    modalias = \"%s\"\n", hd_card->modalias);
+  }
+  else {
+    hd_card->modalias = new_str(module);
+  }
+
+  if(hd) {
+    hd_card->attached_to = hd->idx;
+    hd_card->vendor.id = hd->vendor.id;
+    hd_card->device.id = hd->device.id;
+  }
+  else {
+    hd_card->vendor.name = new_str("Marvell");
+    hd_card->device.name = new_str("Gigabit Ethernet");
+  }
+
+  if(hd_module_is_active(hd_data, module)) {
+    add_str_list(&hd_card->drivers, module);
+  }
+
+  for(hd = hd_data->hd ; hd; hd = hd->next) {
+    if(
+      hd->module == hd_data->module &&
+      hd->base_class.id == bc_network_interface &&
+      hd->sub_class.id == sc_nif_ethernet &&
+      search_str_list(hd->drivers, "mv643xx_eth")
+    ) {
+      hd->attached_to = hd_card->idx;
+
+      for(res = hd->res; res; res = res->next) {
+        if(res->any.type == res_hwaddr) break;
+      }
+
+      if(res) {
+        res2 = new_mem(sizeof *res2);
+        res2->hwaddr.type = res_hwaddr;
+        res2->hwaddr.addr = new_str(res->hwaddr.addr);
+        add_res_entry(&hd_card->res, res2);
+      }
+
+      add_if_name(hd_card, hd);
+
+      break;
+    }
+  }
+}
+
+
 /*
  * iSeries veth devices.
  */
@@ -449,7 +543,10 @@ void add_iseries(hd_data_t *hd_data)
     if(
       hd->module == hd_data->module &&
       hd->base_class.id == bc_network_interface &&
-      search_str_list(hd->drivers, "veth")
+      (
+        search_str_list(hd->drivers, "veth") ||
+        search_str_list(hd->drivers, "iseries_veth")
+      )
     ) {
       hd_card = add_hd_entry(hd_data, __LINE__, 0);
       hd_card->base_class.id = bc_network;
@@ -652,3 +749,5 @@ void add_if_name(hd_t *hd_card, hd_t *hd)
   }
 }
 
+/** @} */
+
